--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity COLSEL is
	port (
		MDSEL		: in  std_logic;
		LOCK_PLL	: in  std_logic;
		CLK100M		: in  std_logic;
		RSTN		: in  std_logic;
		COLORMD		: out std_logic_vector(2 downto 0)
	);
end COLSEL;

architecture RTL of COLSEL is
	signal mdsel_f1		: std_logic;
	signal mdsel_f2		: std_logic;
	signal cnt10ms		: std_logic_vector(19 downto 0);
	signal cntlow		: std_logic_vector(3 downto 0);
	signal cnthigh		: std_logic_vector(3 downto 0);
	signal initrd_f1	: std_logic;
	signal initrd_f2	: std_logic;
	signal initrd_f3	: std_logic;
	signal keyin		: std_logic;
	signal keyin_f1		: std_logic;
	signal colormd_i	: std_logic_vector(2 downto 0);


begin

-- input latch
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			mdsel_f1 <= '0';
			mdsel_f2 <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			mdsel_f1 <= MDSEL;
			mdsel_f2 <= mdsel_f1;
		end if;
	end process;


-- 10ms counter
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			cnt10ms <= (others => '0');
		elsif (CLK100M'event and CLK100M = '1') then
			cnt10ms <= cnt10ms + 1;
		end if;
	end process;

-- chattering cancel
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			initrd_f1 <= '1';
			initrd_f2 <= '1';
			initrd_f3 <= '1';
		elsif (CLK100M'event and CLK100M = '1') then
			if (LOCK_PLL = '1') then
				initrd_f1 <= '0';
				initrd_f2 <= initrd_f1;
				initrd_f3 <= initrd_f2;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			cntlow  <= (others => '0');
			cnthigh <= (others => '0');
		elsif (CLK100M'event and CLK100M = '1') then
			if (initrd_f3 = '1') then
				cntlow  <= (others => '0');
				cnthigh <= (others => '0');
			elsif (cnt10ms = 0) then
				if (mdsel_f2 = '0') then
					cntlow  <= cntlow + 1;
					cnthigh <= (others => '0');
				else
					cntlow  <= (others => '0');
					cnthigh <= cnthigh+ 1;
				end if;
			end if;
		end if;
	end process;

-- mode select input
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			keyin    <= '0';
			keyin_f1 <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (initrd_f3 = '1') then
				keyin <= '0';
			elsif (cnthigh = "1111") then
				keyin <= '1';
			elsif (cntlow = "1111") then
				keyin <= '0';
			end if;
			keyin_f1 <= keyin;
		end if;
	end process;

-- mode select
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			colormd_i <= (others => '0');
		elsif (CLK100M'event and CLK100M = '1') then
			if (initrd_f3 = '1') then
				colormd_i <= (others => '0');
			elsif (keyin = '1' and keyin_f1 = '0') then
				if (colormd_i = "000") then
					colormd_i <= "100";
				else
					colormd_i <= colormd_i + 1;
				end if;
			end if;
		end if;
	end process;


	COLORMD   <= colormd_i;


end RTL;
