--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity DATASFT is
	port (
		HSYNCN		: in  std_logic;
		VSYNCN		: in  std_logic;
		RGBH		: in  std_logic_vector(3 downto 0);
		CLKENB		: in  std_logic;
		SRMODE		: in  std_logic;
		LOCK1		: in  std_logic;
		LOCK2		: in  std_logic;
		CLK100M		: in  std_logic;
		RSTN		: in  std_logic;
		HSYNCN_O	: out std_logic;
		VSYNCN_O	: out std_logic;
		RGBH_O		: out std_logic_vector(3 downto 0);
		LOCK3		: out std_logic;
		LOCK4		: out std_logic
	);
end DATASFT;

architecture RTL of DATASFT is

	signal rgbh_f1		: std_logic_vector(3 downto 0);
	signal rgbh_f2		: std_logic_vector(3 downto 0);
	signal rgbh_f3		: std_logic_vector(3 downto 0);
	signal rgbh_f4		: std_logic_vector(3 downto 0);
	signal rgbh_f5		: std_logic_vector(3 downto 0);
	signal hsyncn_f1	: std_logic;
	signal hsyncn_f2	: std_logic;
	signal hsyncn_f3	: std_logic;
	signal vsyncn_f1	: std_logic;
	signal vsyncn_f2	: std_logic;
	signal vsyncn_f3	: std_logic;
	signal isrgbh_f1	: std_logic;

	signal hcnt			: std_logic_vector(9 downto 0);
	signal vcnt			: std_logic_vector(8 downto 0);
	signal vsyncnf_f1	: std_logic;
	signal vsyncnf_f2	: std_logic;

	signal bord3_0l		: std_logic;
	signal bord3_1l		: std_logic;
	signal bord3_2l		: std_logic;
	signal bord3_3l		: std_logic;
	signal bord3_4l		: std_logic;
	signal bord3_5l		: std_logic;
	signal bord3_0r		: std_logic;
	signal bord3_1r		: std_logic;
	signal bord3_2r		: std_logic;
	signal bord3_3r		: std_logic;
	signal bord3_4r		: std_logic;
	signal bord3_5r		: std_logic;

	signal bord4_0l		: std_logic;
	signal bord4_1l		: std_logic;
	signal bord4_2l		: std_logic;
	signal bord4_3l		: std_logic;
	signal bord4_4l		: std_logic;
	signal bord4_5l		: std_logic;
	signal bord4_0r		: std_logic;
	signal bord4_1r		: std_logic;
	signal bord4_2r		: std_logic;
	signal bord4_3r		: std_logic;
	signal bord4_4r		: std_logic;
	signal bord4_5r		: std_logic;

	signal lock3_i		: std_logic;
	signal lock4_i		: std_logic;

	signal vcntenb		: std_logic;
	signal sftdt3		: std_logic_vector(2 downto 0);
	signal sftdt4		: std_logic_vector(2 downto 0);

	signal sftdt		: std_logic_vector(2 downto 0);

begin

---- data shift
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			rgbh_f1 <= (others => '0');
			rgbh_f2 <= (others => '0');
			rgbh_f3 <= (others => '0');
			rgbh_f4 <= (others => '0');
			rgbh_f5 <= (others => '0');
			hsyncn_f1 <= '1';
			hsyncn_f2 <= '1';
			hsyncn_f3 <= '1';
			vsyncn_f1 <= '1';
			vsyncn_f2 <= '1';
			vsyncn_f3 <= '1';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				rgbh_f1 <= RGBH;
				rgbh_f2 <= rgbh_f1;
				rgbh_f3 <= rgbh_f2;
				rgbh_f4 <= rgbh_f3;
				rgbh_f5 <= rgbh_f4;
				hsyncn_f1 <= HSYNCN;
				hsyncn_f2 <= hsyncn_f1;
				hsyncn_f3 <= hsyncn_f2;
				vsyncn_f1 <= VSYNCN;
				vsyncn_f2 <= vsyncn_f1;
				vsyncn_f3 <= vsyncn_f2;
			end if;
		end if;
	end process;

	isrgbh_f1 <= '0' when (rgbh_f1 = "0000") else '1';

---- horizontal counter
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			hcnt <= (others => '1');
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (HSYNCN = '0' and hsyncn_f1 = '1') then
					hcnt <= (others => '0');
				elsif (hcnt /= "1111111111") then
					hcnt <= hcnt + 1;
				end if;
			end if;
		end if;
	end process;

---- vertical counter
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			vsyncnf_f1 <= '1';
			vsyncnf_f2 <= '1';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (hcnt = 0) then
					vsyncnf_f1 <= VSYNCN;
					vsyncnf_f2 <= vsyncnf_f1;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			vcnt <= (others => '0');
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (hcnt = 1) then
					if (vsyncnf_f1 = '0' and vsyncnf_f2 = '1') then
						vcnt <= (others => '0');
					else
						vcnt <= vcnt + 1;
					end if;
				end if;
			end if;
		end if;
	end process;

-- back porch and left border (for 320x200)
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord3_0l <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 5) then
						bord3_0l <= '0';
					elsif (hcnt > 5 and hcnt < 192-2) then
						if (isrgbh_f1 = '1') then
							bord3_0l <= '1';
						end if;
					end if;
				else
					if (hcnt = 5) then
						bord3_0l <= '0';
					elsif (hcnt > 5 and hcnt < 200-2) then
						if (isrgbh_f1 = '1') then
							bord3_0l <= '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord3_1l <= '0';
			bord3_2l <= '0';
			bord3_3l <= '0';
			bord3_4l <= '0';
			bord3_5l <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 192-2) then
						bord3_1l <= isrgbh_f1;
					end if;
					if (hcnt = 192-1) then
						bord3_2l <= isrgbh_f1;
					end if;
					if (hcnt = 192+0) then
						bord3_3l <= isrgbh_f1;
					end if;
					if (hcnt = 192+1) then
						bord3_4l <= isrgbh_f1;
					end if;
					if (hcnt = 192+2) then
						bord3_5l <= isrgbh_f1;
					end if;
				else
					if (hcnt = 200-2) then
						bord3_1l <= isrgbh_f1;
					end if;
					if (hcnt = 200-1) then
						bord3_2l <= isrgbh_f1;
					end if;
					if (hcnt = 200+0) then
						bord3_3l <= isrgbh_f1;
					end if;
					if (hcnt = 200+1) then
						bord3_4l <= isrgbh_f1;
					end if;
					if (hcnt = 200+2) then
						bord3_5l <= isrgbh_f1;
					end if;
				end if;
			end if;
		end if;
	end process;


-- front porch and right border (for 320x200)
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord3_0r <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 5) then
						bord3_0r <= '0';
					elsif (hcnt > 831+2 and hcnt < 910) then
						if (isrgbh_f1 = '1') then
							bord3_0r <= '1';
						end if;
					end if;
				else
					if (hcnt = 5) then
						bord3_0r <= '0';
					elsif (hcnt > 839+2 and hcnt < 910) then
						if (isrgbh_f1 = '1') then
							bord3_0r <= '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord3_1r <= '0';
			bord3_2r <= '0';
			bord3_3r <= '0';
			bord3_4r <= '0';
			bord3_5r <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 831-2) then
						bord3_1r <= isrgbh_f1;
					end if;
					if (hcnt = 831-1) then
						bord3_2r <= isrgbh_f1;
					end if;
					if (hcnt = 831+0) then
						bord3_3r <= isrgbh_f1;
					end if;
					if (hcnt = 831+1) then
						bord3_4r <= isrgbh_f1;
					end if;
					if (hcnt = 831+2) then
						bord3_5r <= isrgbh_f1;
					end if;
				else
					if (hcnt = 839-2) then
						bord3_1r <= isrgbh_f1;
					end if;
					if (hcnt = 839-1) then
						bord3_2r <= isrgbh_f1;
					end if;
					if (hcnt = 839+0) then
						bord3_3r <= isrgbh_f1;
					end if;
					if (hcnt = 839+1) then
						bord3_4r <= isrgbh_f1;
					end if;
					if (hcnt = 839+2) then
						bord3_5r <= isrgbh_f1;
					end if;
				end if;
			end if;
		end if;
	end process;

-- back porch and left border (for 256x192)
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord4_0l <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 5) then
						bord4_0l <= '0';
					elsif (hcnt > 5 and hcnt < 192+64-2) then
						if (isrgbh_f1 = '1') then
							bord4_0l <= '1';
						end if;
					end if;
				else
					if (hcnt = 5) then
						bord4_0l <= '0';
					elsif (hcnt > 5 and hcnt < 200+64-2) then
						if (isrgbh_f1 = '1') then
							bord4_0l <= '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord4_1l <= '0';
			bord4_2l <= '0';
			bord4_3l <= '0';
			bord4_4l <= '0';
			bord4_5l <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 192+64-2) then
						bord4_1l <= isrgbh_f1;
					end if;
					if (hcnt = 192+64-1) then
						bord4_2l <= isrgbh_f1;
					end if;
					if (hcnt = 192+64+0) then
						bord4_3l <= isrgbh_f1;
					end if;
					if (hcnt = 192+64+1) then
						bord4_4l <= isrgbh_f1;
					end if;
					if (hcnt = 192+64+2) then
						bord4_5l <= isrgbh_f1;
					end if;
				else
					if (hcnt = 200+64-2) then
						bord4_1l <= isrgbh_f1;
					end if;
					if (hcnt = 200+64-1) then
						bord4_2l <= isrgbh_f1;
					end if;
					if (hcnt = 200+64+0) then
						bord4_3l <= isrgbh_f1;
					end if;
					if (hcnt = 200+64+1) then
						bord4_4l <= isrgbh_f1;
					end if;
					if (hcnt = 200+64+2) then
						bord4_5l <= isrgbh_f1;
					end if;
				end if;
			end if;
		end if;
	end process;


-- front porch and right border (for 256x192)
	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord4_0r <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 5) then
						bord4_0r <= '0';
					elsif (hcnt > 831-64+2 and hcnt < 910) then
						if (isrgbh_f1 = '1') then
							bord4_0r <= '1';
						end if;
					end if;
				else
					if (hcnt = 5) then
						bord4_0r <= '0';
					elsif (hcnt > 839-64+2 and hcnt < 910) then
						if (isrgbh_f1 = '1') then
							bord4_0r <= '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			bord4_1r <= '0';
			bord4_2r <= '0';
			bord4_3r <= '0';
			bord4_4r <= '0';
			bord4_5r <= '0';
		elsif (CLK100M'event and CLK100M = '1') then
			if (CLKENB = '1') then
				if (SRMODE = '0') then
					if (hcnt = 831-64-2) then
						bord4_1r <= isrgbh_f1;
					end if;
					if (hcnt = 831-64-1) then
						bord4_2r <= isrgbh_f1;
					end if;
					if (hcnt = 831-64+0) then
						bord4_3r <= isrgbh_f1;
					end if;
					if (hcnt = 831-64+1) then
						bord4_4r <= isrgbh_f1;
					end if;
					if (hcnt = 831-64+2) then
						bord4_5r <= isrgbh_f1;
					end if;
				else
					if (hcnt = 839-64-2) then
						bord4_1r <= isrgbh_f1;
					end if;
					if (hcnt = 839-64-1) then
						bord4_2r <= isrgbh_f1;
					end if;
					if (hcnt = 839-64+0) then
						bord4_3r <= isrgbh_f1;
					end if;
					if (hcnt = 839-64+1) then
						bord4_4r <= isrgbh_f1;
					end if;
					if (hcnt = 839-64+2) then
						bord4_5r <= isrgbh_f1;
					end if;
				end if;
			end if;
		end if;
	end process;


	vcntenb <=
		'1' when (SRMODE = '0' and vcnt >= 33 and vcnt < 233) else
		'1' when (SRMODE = '1' and vcnt >= 37 and vcnt < 241) else
		'0';

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			lock3_i <= '0';
			sftdt3  <= "100";
		elsif (CLK100M'event and CLK100M = '1') then
			if (LOCK1 = '0' or LOCK2 = '0') then
				lock3_i <= '0';
				sftdt3  <= "100";
			elsif (lock3_i = '0') then
				if (CLKENB = '1') then
					if (vcntenb = '1' and hcnt = 910) then

						if (
							bord3_0l = '0' and
							bord3_1l = '1' and
							bord3_1r = '1' and
							bord3_2r = '0' and
							bord3_3r = '0' and
							bord3_4r = '0' and
							bord3_5r = '0' and
							bord3_0r = '0'
						) then
							lock3_i <= '1';
							sftdt3  <= "010";

						elsif (
							bord3_0l = '0' and
							bord3_1l = '0' and
							bord3_2l = '1' and
							bord3_2r = '1' and
							bord3_3r = '0' and
							bord3_4r = '0' and
							bord3_5r = '0' and
							bord3_0r = '0'
						) then
							lock3_i <= '1';
							sftdt3  <= "011";

						elsif (
							bord3_0l = '0' and
							bord3_1l = '0' and
							bord3_2l = '0' and
							bord3_3l = '1' and
							bord3_3r = '1' and
							bord3_4r = '0' and
							bord3_5r = '0' and
							bord3_0r = '0'
						) then
							lock3_i <= '1';
							sftdt3  <= "100";

						elsif (
							bord3_0l = '0' and
							bord3_1l = '0' and
							bord3_2l = '0' and
							bord3_3l = '0' and
							bord3_4l = '1' and
							bord3_4r = '1' and
							bord3_5r = '0' and
							bord3_0r = '0'
						) then
							lock3_i <= '1';
							sftdt3  <= "101";

						elsif (
							bord3_0l = '0' and
							bord3_1l = '0' and
							bord3_2l = '0' and
							bord3_3l = '0' and
							bord3_4l = '0' and
							bord3_5l = '1' and
							bord3_5r = '1' and
							bord3_0r = '0'
						) then
							lock3_i <= '1';
							sftdt3  <= "110";

						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	process (CLK100M,RSTN)
	begin
		if (RSTN = '0') then
			lock4_i <= '0';
			sftdt4  <= "100";
		elsif (CLK100M'event and CLK100M = '1') then
			if (LOCK1 = '0' or LOCK2 = '0') then
				lock4_i <= '0';
				sftdt4  <= "100";
			elsif (lock3_i = '0' and lock4_i = '0') then
				if (CLKENB = '1') then
					if (vcntenb = '1' and hcnt = 910) then

						if (
							bord4_0l = '0' and
							bord4_1l = '1' and
							bord4_1r = '1' and
							bord4_2r = '0' and
							bord4_3r = '0' and
							bord4_4r = '0' and
							bord4_5r = '0' and
							bord4_0r = '0'
						) then
							lock4_i <= '1';
							sftdt4  <= "010";

						elsif (
							bord4_0l = '0' and
							bord4_1l = '0' and
							bord4_2l = '1' and
							bord4_2r = '1' and
							bord4_3r = '0' and
							bord4_4r = '0' and
							bord4_5r = '0' and
							bord4_0r = '0'
						) then
							lock4_i <= '1';
							sftdt4  <= "011";

						elsif (
							bord4_0l = '0' and
							bord4_1l = '0' and
							bord4_2l = '0' and
							bord4_3l = '1' and
							bord4_3r = '1' and
							bord4_4r = '0' and
							bord4_5r = '0' and
							bord4_0r = '0'
						) then
							lock4_i <= '1';
							sftdt4  <= "100";

						elsif (
							bord4_0l = '0' and
							bord4_1l = '0' and
							bord4_2l = '0' and
							bord4_3l = '0' and
							bord4_4l = '1' and
							bord4_4r = '1' and
							bord4_5r = '0' and
							bord4_0r = '0'
						) then
							lock4_i <= '1';
							sftdt4  <= "101";

						elsif (
							bord4_0l = '0' and
							bord4_1l = '0' and
							bord4_2l = '0' and
							bord4_3l = '0' and
							bord4_4l = '0' and
							bord4_5l = '1' and
							bord4_5r = '1' and
							bord4_0r = '0'
						) then
							lock4_i <= '1';
							sftdt4  <= "110";

						end if;
					end if;
				end if;
			end if;
		end if;
	end process;


	sftdt <=
		sftdt3 when (lock3_i = '1') else
		sftdt4 when (lock4_i = '1') else
		"100";




	HSYNCN_O <= hsyncn_f3;
	VSYNCN_O <= vsyncn_f3;

	RGBH_O <=
		rgbh_f1 when (sftdt = "110") else		-- shift = -2
		rgbh_f2 when (sftdt = "101") else		-- shift = -1
		rgbh_f3 when (sftdt = "100") else		-- shift =  0
		rgbh_f4 when (sftdt = "011") else		-- shift = +1
		rgbh_f5 when (sftdt = "010") else		-- shift = +2
		(others => '0');

	LOCK3 <= lock3_i;
	LOCK4 <= lock4_i;

end RTL;
